@@ -49,163 +49,46 @@ uint config_msg_level = CONFIG_ERROR_LEVEL;
 
 #ifdef CONFIG_PATH_AUTO_SELECT
 #ifdef BCMSDIO
-#define BCM4330B2_CONF_NAME "config_40183b2.txt"
-#define BCM43362A0_CONF_NAME "config_40181a0.txt"
-#define BCM43362A2_CONF_NAME "config_40181a2.txt"
-#define BCM43438A0_CONF_NAME "config_43438a0.txt"
-#define BCM43438A1_CONF_NAME "config_43438a1.txt"
-#define BCM43436B0_CONF_NAME "config_43436b0.txt"
-#define BCM4334B1_CONF_NAME "config_4334b1.txt"
-#define BCM43341B0_CONF_NAME "config_43341b0.txt"
-#define BCM43241B4_CONF_NAME "config_43241b4.txt"
-#define BCM4339A0_CONF_NAME "config_4339a0.txt"
-#define BCM43455C0_CONF_NAME "config_43455c0.txt"
-#define BCM4354A1_CONF_NAME "config_4354a1.txt"
+#define CONFIG_BCM4330B2 "config_40183b2.txt"
+#define CONFIG_BCM43362A0 "config_40181a0.txt"
+#define CONFIG_BCM43362A2 "config_40181a2.txt"
+#define CONFIG_BCM43438A0 "config_43438a0.txt"
+#define CONFIG_BCM43438A1 "config_43438a1.txt"
+#define CONFIG_BCM43436B0 "config_43436b0.txt"
+#define CONFIG_BCM4334B1 "config_4334b1.txt"
+#define CONFIG_BCM43341B0 "config_43341b0.txt"
+#define CONFIG_BCM43241B4 "config_43241b4.txt"
+#define CONFIG_BCM4339A0 "config_4339a0.txt"
+#define CONFIG_BCM43455C0 "config_43455c0.txt"
+#define CONFIG_BCM4354A1 "config_4354a1.txt"
 #endif
-#define BCM4356A2_CONF_NAME "config_4356a2.txt"
-#define BCM4359B1_CONF_NAME "config_4359b1.txt"
-#define BCM4359C0_CONF_NAME "config_4359c0.txt"
+#define CONFIG_BCM4356A2 "config_4356a2.txt"
+#define CONFIG_BCM4359B1 "config_4359b1.txt"
+#define CONFIG_BCM4359C0 "config_4359c0.txt"
 #endif
 
 #ifdef BCMSDIO
 #define SBSDIO_CIS_SIZE_LIMIT		0x200		/* maximum bytes in one CIS */
 
-const static char *bcm4330b2_fw_name[] = {
-	"fw_bcm40183b2.bin",
-	"fw_bcm40183b2_apsta.bin",
-	"fw_bcm40183b2_p2p.bin",
-	"fw_bcm40183b2_es.bin",
-	"fw_bcm40183b2_mfg.bin"
-};
-
-const static char *bcm4330b2_ag_fw_name[] = {
-	"fw_bcm40183b2_ag.bin",
-	"fw_bcm40183b2_ag_apsta.bin",
-	"fw_bcm40183b2_ag_p2p.bin",
-	"fw_bcm40183b2_ag_es.bin",
-	"fw_bcm40183b2_ag_mfg.bin"
-};
-
-const static char *bcm43362a0_fw_name[] = {
-	"fw_bcm40181a0.bin",
-	"fw_bcm40181a0_apsta.bin",
-	"fw_bcm40181a0_p2p.bin",
-	"fw_bcm40181a0_es.bin",
-	"fw_bcm40181a0_mfg.bin"
-};
-
-const static char *bcm43362a2_fw_name[] = {
-	"fw_bcm40181a2.bin",
-	"fw_bcm40181a2_apsta.bin",
-	"fw_bcm40181a2_p2p.bin",
-	"fw_bcm40181a2_es.bin",
-	"fw_bcm40181a2_mfg.bin"
-};
-
-const static char *bcm4334b1_ag_fw_name[] = {
-	"fw_bcm4334b1_ag.bin",
-	"fw_bcm4334b1_ag_apsta.bin",
-	"fw_bcm4334b1_ag_p2p.bin",
-	"fw_bcm4334b1_ag_es.bin",
-	"fw_bcm4334b1_ag_mfg.bin"
-};
-
-const static char *bcm43438a0_fw_name[] = {
-	"fw_bcm43438a0.bin",
-	"fw_bcm43438a0_apsta.bin",
-	"fw_bcm43438a0_p2p.bin",
-	"fw_bcm43438a0_es.bin",
-	"fw_bcm43438a0_mfg.bin"
-};
-
-const static char *bcm43438a1_fw_name[] = {
-	"fw_bcm43438a1.bin",
-	"fw_bcm43438a1_apsta.bin",
-	"fw_bcm43438a1_p2p.bin",
-	"fw_bcm43438a1_es.bin",
-	"fw_bcm43438a1_mfg.bin"
-};
-
-const static char *bcm43436b0_fw_name[] = {
-	"fw_bcm43436b0.bin",
-	"fw_bcm43436b0_apsta.bin",
-	"fw_bcm43436b0_p2p.bin",
-	"fw_bcm43436b0_es.bin",
-	"fw_bcm43436b0_mfg.bin"
-};
-
-const static char *bcm43341b0_ag_fw_name[] = {
-	"fw_bcm43341b0_ag.bin",
-	"fw_bcm43341b0_ag_apsta.bin",
-	"fw_bcm43341b0_ag_p2p.bin",
-	"fw_bcm43341b0_ag_es.bin",
-	"fw_bcm43341b0_ag_mfg.bin"
-};
-
-const static char *bcm43241b4_ag_fw_name[] = {
-	"fw_bcm43241b4_ag.bin",
-	"fw_bcm43241b4_ag_apsta.bin",
-	"fw_bcm43241b4_ag_p2p.bin",
-	"fw_bcm43241b4_ag_es.bin",
-	"fw_bcm43241b4_ag_mfg.bin"
-};
-
-const static char *bcm4339a0_ag_fw_name[] = {
-	"fw_bcm4339a0_ag.bin",
-	"fw_bcm4339a0_ag_apsta.bin",
-	"fw_bcm4339a0_ag_p2p.bin",
-	"fw_bcm4339a0_ag_es.bin",
-	"fw_bcm4339a0_ag_mfg.bin"
-};
-
-const static char *bcm43455c0_ag_fw_name[] = {
-	"fw_bcm43455c0_ag.bin",
-	"fw_bcm43455c0_ag_apsta.bin",
-	"fw_bcm43455c0_ag_p2p.bin",
-	"fw_bcm43455c0_ag_es.bin",
-	"fw_bcm43455c0_ag_mfg.bin"
-};
-
-const static char *bcm4354a1_ag_fw_name[] = {
-	"fw_bcm4354a1_ag.bin",
-	"fw_bcm4354a1_ag_apsta.bin",
-	"fw_bcm4354a1_ag_p2p.bin",
-	"fw_bcm4354a1_ag_es.bin",
-	"fw_bcm4354a1_ag_mfg.bin"
-};
-
-const static char *bcm4356a2_ag_fw_name[] = {
-	"fw_bcm4356a2_ag.bin",
-	"fw_bcm4356a2_ag_apsta.bin",
-	"fw_bcm4356a2_ag_p2p.bin",
-	"fw_bcm4356a2_ag_es.bin",
-	"fw_bcm4356a2_ag_mfg.bin"
-};
-
-const static char *bcm4359b1_ag_fw_name[] = {
-	"fw_bcm4359b1_ag.bin",
-	"fw_bcm4359b1_ag_apsta.bin",
-	"fw_bcm4359b1_ag_p2p.bin",
-	"fw_bcm4359b1_ag_es.bin",
-	"fw_bcm4359b1_ag_mfg.bin"
-};
-
-const static char *bcm4359c0_ag_fw_name[] = {
-	"fw_bcm4359c0_ag.bin",
-	"fw_bcm4359c0_ag_apsta.bin",
-	"fw_bcm4359c0_ag_p2p.bin",
-	"fw_bcm4359c0_ag_es.bin",
-	"fw_bcm4359c0_ag_mfg.bin"
-};
+#define FW_BCM4330B2 "fw_bcm40183b2"
+#define FW_BCM4330B2_AG "fw_bcm40183b2_ag"
+#define FW_BCM43362A0 "fw_bcm40181a0"
+#define FW_BCM43362A2 "fw_bcm40181a2"
+#define FW_BCM4334B1 "fw_bcm4334b1_ag"
+#define FW_BCM43438A0 "fw_bcm43438a0"
+#define FW_BCM43438A1 "fw_bcm43438a1"
+#define FW_BCM43436B0 "fw_bcm43436b0"
+#define FW_BCM43341B1 "fw_bcm43341b1_ag"
+#define FW_BCM43241B4 "fw_bcm43241b4_ag"
+#define FW_BCM4339A0 "fw_bcm4339a0_ag"
+#define FW_BCM43455C0 "fw_bcm43455c0_ag"
+#define FW_BCM4354A1 "fw_bcm4354a1_ag"
+#define FW_BCM4356A2 "fw_bcm4356a2_ag"
+#define FW_BCM4359B1 "fw_bcm4359b1_ag"
+#define FW_BCM4359C0 "fw_bcm4359c0_ag"
 #endif
 #ifdef BCMPCIE
-const static char *bcm4356a2_pcie_ag_fw_name[] = {
-	"fw_bcm4356a2_pcie_ag.bin",
-	"fw_bcm4356a2_pcie_ag_apsta.bin",
-	"fw_bcm4356a2_pcie_ag_p2p.bin",
-	"fw_bcm4356a2_pcie_ag_es.bin",
-	"fw_bcm4356a2_pcie_ag_mfg.bin"
-};
+#define FW_BCM4356A2 "fw_bcm4356a2_pcie_ag"
 #endif
 
 #define htod32(i) i
@@ -463,6 +346,7 @@ dhd_conf_set_fw_name_by_chip(dhd_pub_t *dhd, char *fw_path)
 	int fw_type, ag_type;
 	uint chip, chiprev;
 	int i;
+	char fw_tail[10];
 
 	chip = dhd->conf->chip;
 	chiprev = dhd->conf->chiprev;
@@ -498,84 +382,95 @@ dhd_conf_set_fw_name_by_chip(dhd_pub_t *dhd, char *fw_path)
 		(strstr(&fw_path[i], "_es") ? FW_TYPE_ES :
 		FW_TYPE_STA))));
 
+	if (fw_type == FW_TYPE_STA)
+		strcpy(fw_tail, ".bin");
+	else if (fw_type == FW_TYPE_APSTA)
+		strcpy(fw_tail, "_apsta.bin");
+	else if (fw_type == FW_TYPE_P2P)
+		strcpy(fw_tail, "_p2p.bin");
+	else if (fw_type == FW_TYPE_ES)
+		strcpy(fw_tail, "_es.bin");
+	else if (fw_type == FW_TYPE_MFG)
+		strcpy(fw_tail, "_mfg.bin");
+
 	switch (chip) {
 #ifdef BCMSDIO
 		case BCM4330_CHIP_ID:
 			if (ag_type == FW_TYPE_G) {
 				if (chiprev == BCM4330B2_CHIP_REV)
-					strcpy(&fw_path[i+1], bcm4330b2_fw_name[fw_type]);
-				break;
+					strcpy(&fw_path[i+1], FW_BCM4330B2);
 			} else {
 				if (chiprev == BCM4330B2_CHIP_REV)
-					strcpy(&fw_path[i+1], bcm4330b2_ag_fw_name[fw_type]);
-				break;
+					strcpy(&fw_path[i+1], FW_BCM4330B2_AG);
 			}
+			break;
 		case BCM43362_CHIP_ID:
 			if (chiprev == BCM43362A0_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm43362a0_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM43362A0);
 			else
-				strcpy(&fw_path[i+1], bcm43362a2_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM43362A2);
 			break;
 		case BCM43430_CHIP_ID:
 			if (chiprev == BCM43430A0_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm43438a0_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM43438A0);
 			else if (chiprev == BCM43430A1_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm43438a1_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM43438A1);
 			else if (chiprev == BCM43430A2_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm43436b0_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM43436B0);
 			break;
 		case BCM4334_CHIP_ID:
 			if (chiprev == BCM4334B1_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4334b1_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM4334B1);
 			break;
 		case BCM43340_CHIP_ID:
 		case BCM43341_CHIP_ID:
 			if (chiprev == BCM43341B0_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm43341b0_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM43341B1);
 			break;
 		case BCM4324_CHIP_ID:
 			if (chiprev == BCM43241B4_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm43241b4_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM43241B4);
 			break;
 		case BCM4335_CHIP_ID:
 			if (chiprev == BCM4335A0_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4339a0_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM4339A0);
+			break;
+		case BCM4339_CHIP_ID:
+			if (chiprev == BCM4339A0_CHIP_REV)
+				strcpy(&fw_path[i+1], FW_BCM4339A0);
 			break;
 		case BCM4345_CHIP_ID:
 		case BCM43454_CHIP_ID:
 			if (chiprev == BCM43455C0_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm43455c0_ag_fw_name[fw_type]);
-			break;
-		case BCM4339_CHIP_ID:
-			if (chiprev == BCM4339A0_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4339a0_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM43455C0);
 			break;
 		case BCM4354_CHIP_ID:
 			if (chiprev == BCM4354A1_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4354a1_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM4354A1);
 			else if (chiprev == BCM4356A2_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4356a2_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM4356A2);
 			break;
 		case BCM4356_CHIP_ID:
 		case BCM4371_CHIP_ID:
 			if (chiprev == BCM4356A2_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4356a2_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM4356A2);
 			break;
 		case BCM4359_CHIP_ID:
 			if (chiprev == BCM4359B1_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4359b1_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM4359B1);
 			else if (chiprev == BCM4359C0_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4359c0_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM4359C0);
 			break;
 #endif
 #ifdef BCMPCIE
 		case BCM4354_CHIP_ID:
 		case BCM4356_CHIP_ID:
 			if (chiprev == BCM4356A2_CHIP_REV)
-				strcpy(&fw_path[i+1], bcm4356a2_pcie_ag_fw_name[fw_type]);
+				strcpy(&fw_path[i+1], FW_BCM4356A2);
 			break;
 #endif
 	}
+	strcat(fw_path, fw_tail);
 
 	printf("%s: firmware_path=%s\n", __FUNCTION__, fw_path);
 }
@@ -677,71 +572,71 @@ dhd_conf_set_conf_name_by_chip(dhd_pub_t *dhd, char *conf_path)
 #ifdef BCMSDIO
 		case BCM4330_CHIP_ID:
 			if (chiprev == BCM4330B2_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4330B2_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4330B2);
 			break;
 		case BCM43362_CHIP_ID:
 			if (chiprev == BCM43362A0_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM43362A0_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM43362A0);
 			else
-				strcpy(&conf_path[i+1], BCM43362A2_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM43362A2);
 			break;
 		case BCM43430_CHIP_ID:
 			if (chiprev == BCM43430A0_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM43438A0_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM43438A0);
 			else if (chiprev == BCM43430A1_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM43438A1_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM43438A1);
 			else if (chiprev == BCM43430A2_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM43436B0_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM43436B0);
 			break;
 		case BCM4334_CHIP_ID:
 			if (chiprev == BCM4334B1_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4334B1_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4334B1);
 			break;
 		case BCM43340_CHIP_ID:
 		case BCM43341_CHIP_ID:
 			if (chiprev == BCM43341B0_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM43341B0_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM43341B0);
 			break;
 		case BCM4324_CHIP_ID:
 			if (chiprev == BCM43241B4_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM43241B4_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM43241B4);
 			break;
 		case BCM4335_CHIP_ID:
 			if (chiprev == BCM4335A0_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4339A0_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4339A0);
 			break;
 		case BCM4345_CHIP_ID:
 		case BCM43454_CHIP_ID:
 			if (chiprev == BCM43455C0_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM43455C0_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM43455C0);
 			break;
 		case BCM4339_CHIP_ID:
 			if (chiprev == BCM4339A0_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4339A0_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4339A0);
 			break;
 		case BCM4354_CHIP_ID:
 			if (chiprev == BCM4354A1_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4354A1_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4354A1);
 			else if (chiprev == BCM4356A2_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4356A2_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4356A2);
 			break;
 		case BCM4356_CHIP_ID:
 		case BCM4371_CHIP_ID:
 			if (chiprev == BCM4356A2_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4356A2_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4356A2);
 			break;
 		case BCM4359_CHIP_ID:
 			if (chiprev == BCM4359B1_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4359B1_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4359B1);
 			else if (chiprev == BCM4359C0_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4359C0_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4359C0);
 			break;
 #endif
 #ifdef BCMPCIE
 		case BCM4354_CHIP_ID:
 		case BCM4356_CHIP_ID:
 			if (chiprev == BCM4356A2_CHIP_REV)
-				strcpy(&conf_path[i+1], BCM4356A2_CONF_NAME);
+				strcpy(&conf_path[i+1], CONFIG_BCM4356A2);
 			break;
 #endif
 	}
@@ -1130,11 +1025,16 @@ dhd_conf_add_pkt_filter(dhd_pub_t *dhd)
 #define MACS "%02x%02x%02x%02x%02x%02x"
 
 	/*
-	 * Filter out all pkt: actually not to enable this since 4-way handshake will be filter out as well.
-	 * 1) pkt_filter_mode=0
-	 * 2) pkt_filter_add=99 0 0 0 0x000000000000 0x000000000000
-	 * Netbios pkt: 120 0 0 12 0xFFFF000000000000000000FF000000000000000000000000FFFF 0x0800000000000000000000110000000000000000000000000089
-	 * magic_pkt_filter_add=141 0 1 12
+	 * 1. Filter out all pkt: actually not to enable this since 4-way handshake will be filter out as well.
+	 *   1) dhd_master_mode=0
+	 *   2) pkt_filter_add=99 0 0 0 0x000000000000 0x000000000000
+	 * 2. Filter in less pkt: ARP(0x0806, ID is 105), BRCM(0x886C), 802.1X(0x888E)
+	 *   1) dhd_master_mode=1
+	 *   2) pkt_filter_del=100, 102, 103, 104, 105
+	 *   3) pkt_filter_add=131 0 0 12 0xFFFF 0x886C, 132 0 0 12 0xFFFF 0x888E
+	 * 3. magic pkt: magic_pkt_filter_add=141 0 1 12
+	 * 4. Filter out netbios pkt:
+	 *   Netbios: 121 0 0 12 0xFFFF000000000000000000FF000000000000000000000000FFFF 0x0800000000000000000000110000000000000000000000000089
 	 */
 	for(i=0; i<dhd->conf->pkt_filter_add.count; i++) {
 		dhd->pktfilter[i+dhd->pktfilter_count] = dhd->conf->pkt_filter_add.filter[i];
@@ -1196,19 +1096,12 @@ dhd_conf_discard_pkt_filter(dhd_pub_t *dhd)
 	dhd->pktfilter[dhd->pktfilter_count] = "112 0 0 12 0xFFFF000000000000000000000000000000000000000000000000FF 0x86DD000000000000000000000000000000000000000000000000FF";
 	dhd->pktfilter_count++;
 	/* discard Netbios pkt */
-	dhd->pktfilter[dhd->pktfilter_count] = "120 0 0 12 0xFFFF000000000000000000FF000000000000000000000000FFFF 0x0800000000000000000000110000000000000000000000000089";
+	dhd->pktfilter[dhd->pktfilter_count] = "121 0 0 12 0xFFFF000000000000000000FF000000000000000000000000FFFF 0x0800000000000000000000110000000000000000000000000089";
 	dhd->pktfilter_count++;
 
 }
 #endif /* PKT_FILTER_SUPPORT */
 
-void
-dhd_conf_set_disable_proptx(dhd_pub_t *dhd)
-{
-	printf("%s: set disable_proptx %d\n", __FUNCTION__, dhd->conf->disable_proptx);
-	disable_proptx = dhd->conf->disable_proptx;
-}
-
 int
 dhd_conf_get_pm(dhd_pub_t *dhd)
 {
@@ -1217,6 +1110,56 @@ dhd_conf_get_pm(dhd_pub_t *dhd)
 	return -1;
 }
 
+#ifdef PROP_TXSTATUS
+int
+dhd_conf_get_disable_proptx(dhd_pub_t *dhd)
+{
+	struct dhd_conf *conf = dhd->conf;
+	int disable_proptx = -1;
+	int fw_proptx = 0;
+
+	/* check fw proptx priority:
+	  * 1st: check fw support by wl cap
+	  * 2nd: 4334/43340/43341/43241 support proptx but not show in wl cap, so enable it by default
+	  * 	   if you would like to disable it, please set disable_proptx=1 in config.txt
+	  * 3th: disable when proptxstatus not support in wl cap
+	  */
+	if (FW_SUPPORTED(dhd, proptxstatus)) {
+		fw_proptx = 1;
+	} else if (conf->chip == BCM4334_CHIP_ID || conf->chip == BCM43340_CHIP_ID ||
+			dhd->conf->chip == BCM43340_CHIP_ID || conf->chip == BCM4324_CHIP_ID) {
+		fw_proptx = 1;
+	} else {
+		fw_proptx = 0;
+	}
+
+	/* returned disable_proptx value:
+	  * -1: disable in STA and enable in P2P(follow original dhd settings when PROP_TXSTATUS_VSDB enabled)
+	  * 0: depend on fw support
+	  * 1: always disable proptx
+	  */
+	if (conf->disable_proptx == 0) {
+		// check fw support as well
+		if (fw_proptx)
+			disable_proptx = 0;
+		else
+			disable_proptx = 1;
+	} else if (conf->disable_proptx >= 1) {
+		disable_proptx = 1;
+	} else {
+		// check fw support as well
+		if (fw_proptx)
+			disable_proptx = -1;
+		else
+			disable_proptx = 1;
+	}
+
+	printf("%s: fw_proptx=%d, disable_proptx=%d\n", __FUNCTION__, fw_proptx, disable_proptx);
+
+	return disable_proptx;
+}
+#endif
+
 unsigned int
 process_config_vars(char *varbuf, unsigned int len, char *pickbuf, char *param)
 {
@@ -1750,6 +1693,54 @@ dhd_conf_read_country_list(dhd_pub_t *dhd, char *bufp, uint len)
 		MFREE(dhd->osh, pick, MAXSZ_BUF);
 }
 
+#ifdef IAPSTA_PREINIT
+/*
+ * iapsta_init=mode [sta|ap|apsta|dualap] vifname [wlan1]
+ * iapsta_config=ifname [wlan0|wlan1] ssid [xxx] chan [x]
+		 hidden [y|n] maxassoc [x]
+		 amode [open|shared|wpapsk|wpa2psk|wpawpa2psk]
+		 emode [none|wep|tkip|aes|tkipaes]
+		 key [xxxxx]
+ * iapsta_enable=ifname [wlan0|wlan1]
+*/
+void
+dhd_conf_read_iapsta(dhd_pub_t *dhd, char *bufp, uint len)
+{
+	uint len_val;
+	char *pick;
+	struct dhd_conf *conf = dhd->conf;
+
+	pick = MALLOC(dhd->osh, MAXSZ_BUF);
+	if (!pick) {
+		CONFIG_ERROR(("%s: Failed to allocate memory %d bytes\n",
+			__FUNCTION__, MAXSZ_BUF));
+		return;
+	}
+
+	memset(pick, 0, MAXSZ_BUF);
+	len_val = process_config_vars(bufp, len, pick, "iapsta_init=");
+	if (len_val) {
+		sprintf(conf->iapsta_init, "iapsta_init %s", pick);
+		printf("%s: iapsta_init=%s\n", __FUNCTION__, conf->iapsta_init);
+	}
+	memset(pick, 0, MAXSZ_BUF);
+	len_val = process_config_vars(bufp, len, pick, "iapsta_config=");
+	if (len_val) {
+		sprintf(conf->iapsta_config, "iapsta_config %s", pick);
+		printf("%s: iapsta_config=%s\n", __FUNCTION__, conf->iapsta_config);
+	}
+	memset(pick, 0, MAXSZ_BUF);
+	len_val = process_config_vars(bufp, len, pick, "iapsta_enable=");
+	if (len_val) {
+		sprintf(conf->iapsta_enable, "iapsta_enable %s", pick);
+		printf("%s: iapsta_enable=%s\n", __FUNCTION__, conf->iapsta_enable);
+	}
+
+	if (pick)
+		MFREE(dhd->osh, pick, MAXSZ_BUF);
+}
+#endif
+
 int
 dhd_conf_read_config(dhd_pub_t *dhd, char *conf_path)
 {
@@ -1805,6 +1796,9 @@ dhd_conf_read_config(dhd_pub_t *dhd, char *conf_path)
 		dhd_conf_read_nv_by_mac(dhd, bufp, len);
 		dhd_conf_read_nv_by_chip(dhd, bufp, len);
 		dhd_conf_read_country_list(dhd, bufp, len);
+#ifdef IAPSTA_PREINIT
+		dhd_conf_read_iapsta(dhd, bufp, len);
+#endif
 
 		/* Process band:
 		 * band=a for 5GHz only and band=b for 2.4GHz only
@@ -1924,6 +1918,17 @@ dhd_conf_read_config(dhd_pub_t *dhd, char *conf_path)
 				dhd_slpauto = TRUE;
 			printf("%s: dhd_slpauto = %d\n", __FUNCTION__, dhd_slpauto);
 		}
+
+		/* Process kso_enable parameters */
+		memset(pick, 0, MAXSZ_BUF);
+		len_val = process_config_vars(bufp, len, pick, "kso_enable=");
+		if (len_val) {
+			if (!strncmp(pick, "1", len_val))
+				dhd_slpauto = FALSE;
+			else
+				dhd_slpauto = TRUE;
+			printf("%s: dhd_slpauto = %d\n", __FUNCTION__, dhd_slpauto);
+		}
 #endif
 
 		/* Process dhd_master_mode parameters */
@@ -2066,17 +2071,6 @@ dhd_conf_read_config(dhd_pub_t *dhd, char *conf_path)
 		}
 
 #ifdef BCMSDIO
-		/* Process kso_enable parameters */
-		memset(pick, 0, MAXSZ_BUF);
-		len_val = process_config_vars(bufp, len, pick, "kso_enable=");
-		if (len_val) {
-			if (!strncmp(pick, "0", len_val))
-				conf->kso_enable = FALSE;
-			else
-				conf->kso_enable = TRUE;
-			printf("%s: kso_enable = %d\n", __FUNCTION__, conf->kso_enable);
-		}
-
 		/* Process bus:txglom */
 		memset(pick, 0, MAXSZ_BUF);
 		len_val = process_config_vars(bufp, len, pick, "bus:txglom=");
@@ -2093,6 +2087,14 @@ dhd_conf_read_config(dhd_pub_t *dhd, char *conf_path)
 			printf("%s: use_rxchain = %d\n", __FUNCTION__, conf->use_rxchain);
 		}
 
+		/* Process dhd_txminmax parameters */
+		memset(pick, 0, MAXSZ_BUF);
+		len_val = process_config_vars(bufp, len, pick, "dhd_txminmax=");
+		if (len_val) {
+			conf->dhd_txminmax = (uint)simple_strtol(pick, NULL, 10);
+			printf("%s: dhd_txminmax = %d\n", __FUNCTION__, conf->dhd_txminmax);
+		}
+
 		/* Process txinrx_thres parameters */
 		memset(pick, 0, MAXSZ_BUF);
 		len_val = process_config_vars(bufp, len, pick, "txinrx_thres=");
@@ -2101,6 +2103,25 @@ dhd_conf_read_config(dhd_pub_t *dhd, char *conf_path)
 			printf("%s: txinrx_thres = %d\n", __FUNCTION__, conf->txinrx_thres);
 		}
 
+		/* Process sd_f2_blocksize parameters */
+		memset(pick, 0, MAXSZ_BUF);
+		len_val = process_config_vars(bufp, len, pick, "sd_f2_blocksize=");
+		if (len_val) {
+			conf->sd_f2_blocksize = (int)simple_strtol(pick, NULL, 10);
+			printf("%s: sd_f2_blocksize = %d\n", __FUNCTION__, conf->sd_f2_blocksize);
+		}
+
+		/* Process oob_enabled_later parameters */
+		memset(pick, 0, MAXSZ_BUF);
+		len_val = process_config_vars(bufp, len, pick, "oob_enabled_later=");
+		if (len_val) {
+			if (!strncmp(pick, "0", len_val))
+				conf->oob_enabled_later = FALSE;
+			else
+				conf->oob_enabled_later = TRUE;
+			printf("%s: oob_enabled_later = %d\n", __FUNCTION__, conf->oob_enabled_later);
+		}
+
 #if defined(BCMSDIOH_TXGLOM)
 		/* Process txglomsize parameters */
 		memset(pick, 0, MAXSZ_BUF);
@@ -2250,6 +2271,14 @@ dhd_conf_read_config(dhd_pub_t *dhd, char *conf_path)
 			printf("%s: PM = %d\n", __FUNCTION__, conf->pm);
 		}
 
+		/* Process pm_in_suspend parameters */
+		memset(pick, 0, MAXSZ_BUF);
+		len_val = process_config_vars(bufp, len, pick, "pm_in_suspend=");
+		if (len_val) {
+			conf->pm_in_suspend = (int)simple_strtol(pick, NULL, 10);
+			printf("%s: pm_in_suspend = %d\n", __FUNCTION__, conf->pm_in_suspend);
+		}
+
 		/* Process pm2_sleep_ret parameters */
 		memset(pick, 0, MAXSZ_BUF);
 		len_val = process_config_vars(bufp, len, pick, "pm2_sleep_ret=");
@@ -2319,6 +2348,27 @@ dhd_conf_read_config(dhd_pub_t *dhd, char *conf_path)
 			printf("%s: num_different_channels = %d\n", __FUNCTION__, conf->num_different_channels);
 		}
 
+		/* Process xmit_in_suspend parameters */
+		memset(pick, 0, MAXSZ_BUF);
+		len_val = process_config_vars(bufp, len, pick, "xmit_in_suspend=");
+		if (len_val) {
+			if (!strncmp(pick, "1", len_val))
+				conf->xmit_in_suspend = TRUE;
+			else
+				conf->xmit_in_suspend = FALSE;
+			printf("%s: xmit_in_suspend = %d\n", __FUNCTION__, conf->xmit_in_suspend);
+		}
+
+#ifdef IDHCPC
+		/* Process dhcpc_enable parameters */
+		memset(pick, 0, MAXSZ_BUF);
+		len_val = process_config_vars(bufp, len, pick, "dhcpc_enable=");
+		if (len_val) {
+			conf->dhcpc_enable = (int)simple_strtol(pick, NULL, 10);
+			printf("%s: dhcpc_enable = %d\n", __FUNCTION__, conf->dhcpc_enable);
+		}
+#endif
+
 		/* Process tsq parameters */
 		memset(pick, 0, MAXSZ_BUF);
 		len_val = process_config_vars(bufp, len, pick, "tsq=");
@@ -2408,13 +2458,14 @@ dhd_conf_set_txglom_params(dhd_pub_t *dhd, bool enable)
 		conf->txglomsize = 0;
 		conf->deferred_tx_len = 0;
 	}
-	printf("%s: swtxglom=%d, txglom_ext=%d\n", __FUNCTION__,
-		conf->swtxglom, conf->txglom_ext);
-	printf("%s: txglom_bucket_size=%d\n", __FUNCTION__, conf->txglom_bucket_size);
+	printf("%s: swtxglom=%d, txglom_ext=%d, txglom_bucket_size=%d\n", __FUNCTION__,
+		conf->swtxglom, conf->txglom_ext, conf->txglom_bucket_size);
 	printf("%s: txglomsize=%d, deferred_tx_len=%d, bus_txglom=%d\n", __FUNCTION__,
 		conf->txglomsize, conf->deferred_tx_len, conf->bus_txglom);
-	printf("%s: tx_in_rx=%d, tx_max_offset=%d, txctl_tmo_fix=%d\n", __FUNCTION__,
-		conf->tx_in_rx, conf->tx_max_offset, conf->txctl_tmo_fix);
+	printf("%s: tx_in_rx=%d, txinrx_thres=%d, dhd_txminmax=%d\n", __FUNCTION__,
+		conf->tx_in_rx, conf->txinrx_thres, conf->dhd_txminmax);
+	printf("%s: tx_max_offset=%d, txctl_tmo_fix=%d\n", __FUNCTION__,
+		conf->tx_max_offset, conf->txctl_tmo_fix);
 
 }
 #endif
@@ -2493,9 +2544,8 @@ dhd_conf_preinit(dhd_pub_t *dhd)
 	conf->spect = -1;
 	conf->txbf = -1;
 	conf->lpc = -1;
-	conf->disable_proptx = 0;
+	conf->disable_proptx = -1;
 #ifdef BCMSDIO
-	conf->kso_enable = TRUE;
 	conf->bus_txglom = -1;
 	conf->use_rxchain = 0;
 	conf->bus_rxglom = TRUE;
@@ -2507,7 +2557,10 @@ dhd_conf_preinit(dhd_pub_t *dhd)
 	conf->tx_in_rx = TRUE;
 	conf->txglom_mode = SDPCM_TXGLOM_MDESC;
 	conf->deferred_tx_len = 0;
+	conf->dhd_txminmax = 1;
 	conf->txinrx_thres = -1;
+	conf->sd_f2_blocksize = 0;
+	conf->oob_enabled_later = FALSE;
 #endif
 	conf->ampdu_ba_wsize = 0;
 	conf->ampdu_hostreorder = -1;
@@ -2515,8 +2568,13 @@ dhd_conf_preinit(dhd_pub_t *dhd)
 	conf->frameburst = -1;
 	conf->deepsleep = FALSE;
 	conf->pm = -1;
+	conf->pm_in_suspend = -1;
 	conf->pm2_sleep_ret = -1;
 	conf->num_different_channels = -1;
+	conf->xmit_in_suspend = TRUE;
+#ifdef IDHCPC
+	conf->dhcpc_enable = -1;
+#endif
 	conf->tsq = 0;
 #ifdef DHDTCPACK_SUPPRESS
 	conf->tcpack_sup_mode = TCPACK_SUP_OFF;
@@ -2525,25 +2583,24 @@ dhd_conf_preinit(dhd_pub_t *dhd)
 	conf->rsdb_mode = -2;
 	conf->vhtmode = -1;
 #ifdef BCMSDIO
-	if ((conf->chip == BCM43362_CHIP_ID) || (conf->chip == BCM4330_CHIP_ID)) {
-		conf->disable_proptx = 1;
-	}
 	if (conf->chip == BCM43430_CHIP_ID || conf->chip == BCM4345_CHIP_ID) {
 		conf->txctl_tmo_fix = 1;
 	}
 #endif
 	if (conf->chip == BCM4354_CHIP_ID || conf->chip == BCM4356_CHIP_ID ||
 			conf->chip == BCM4371_CHIP_ID || conf->chip == BCM4359_CHIP_ID) {
-//		conf->disable_proptx = 1;
 #ifdef DHDTCPACK_SUPPRESS
-		conf->tcpack_sup_mode = 1;
+		conf->tcpack_sup_mode = TCPACK_SUP_REPLACE;
 #endif
 		dhd_rxbound = 64;
 		dhd_txbound = 64;
 		conf->txbf = 1;
 		conf->frameburst = 1;
 #ifdef BCMSDIO
+		conf->dhd_txminmax = -1;
 		conf->txinrx_thres = 128;
+		conf->sd_f2_blocksize = 256;
+		conf->oob_enabled_later = TRUE;
 #endif
 	}
 
