@@ -39,7 +39,7 @@
 #include <dhd_bus.h>
 #include <dhd_linux.h>
 #include <wl_android.h>
-#if defined(CONFIG_WIFI_CONTROL_FUNC) && !defined(USE_ACPI)
+#if defined(CONFIG_WIFI_CONTROL_FUNC) && !defined(CUSTOMER_HW_INTEL)
 #include <linux/wlan_plat.h>
 #endif
 #ifdef CONFIG_DTS
@@ -47,7 +47,7 @@
 #include<linux/of_gpio.h>
 #endif /* CONFIG_DTS */
 
-#if defined(CUSTOMER_HW) || defined(USE_ACPI)
+#if defined(CUSTOMER_HW) || defined(CUSTOMER_HW_INTEL)
 extern int dhd_wlan_init_plat_data(void);
 extern void dhd_wlan_deinit_plat_data(wifi_adapter_info_t *adapter);
 #endif /* CUSTOMER_HW */
@@ -284,14 +284,14 @@ static int wifi_plat_dev_drv_probe(struct platform_device *pdev)
 	ASSERT(dhd_wifi_platdata != NULL);
 	ASSERT(dhd_wifi_platdata->num_adapters == 1);
 	adapter = &dhd_wifi_platdata->adapters[0];
-#if defined(USE_ACPI)
+#if defined(CUSTOMER_HW_INTEL)
 	adapter->wifi_plat_data = (void *)&dhd_wlan_control;
 	resource = &dhd_wlan_resources;
-	dhd_wlan_init_plat_data();
-	if(resource){
-		adapter->irq_num = resource->start;
-		adapter->intr_flags = resource->flags & IRQF_TRIGGER_MASK;
-	}
+	wifi_plat_dev_probe_ret = dhd_wlan_init_plat_data();
+	if (wifi_plat_dev_probe_ret)
+		return wifi_plat_dev_probe_ret;
+	adapter->irq_num = resource->start;
+	adapter->intr_flags = resource->flags & IRQF_TRIGGER_MASK;
 #else
 	adapter->wifi_plat_data = (struct wifi_platform_data *)(pdev->dev.platform_data);
 
@@ -357,7 +357,7 @@ static int wifi_plat_dev_drv_remove(struct platform_device *pdev)
 #ifdef CONFIG_DTS
 	regulator_put(wifi_regulator);
 #endif
-#if defined(USE_ACPI)
+#if defined(CUSTOMER_HW_INTEL)
 	dhd_wlan_deinit_plat_data(adapter);
 #endif /* CONFIG_DTS */
 	return 0;
@@ -500,6 +500,8 @@ static int wifi_ctrlfunc_register_drv(void)
 		resource = &dhd_wlan_resources;
 #ifdef CUSTOMER_HW
 		wifi_plat_dev_probe_ret = dhd_wlan_init_plat_data();
+		if (wifi_plat_dev_probe_ret)
+			return wifi_plat_dev_probe_ret;
 #endif
 		adapter->irq_num = resource->start;
 		adapter->intr_flags = resource->flags & IRQF_TRIGGER_MASK;
@@ -552,9 +554,9 @@ void wifi_ctrlfunc_unregister_drv(void)
 	}
 #endif /* !defined(CONFIG_DTS) */
 
-//#if defined(CUSTOMER_HW)
+#if defined(CUSTOMER_HW)
 	dhd_wlan_deinit_plat_data(adapter);
-//#endif
+#endif
 
 	kfree(dhd_wifi_platdata->adapters);
 	dhd_wifi_platdata->adapters = NULL;
@@ -743,7 +745,7 @@ static int dhd_wifi_platform_load_sdio(void)
 	int i;
 	int err = 0;
 	wifi_adapter_info_t *adapter;
-	printk(KERN_INFO"enter %s:%d\n", __func__, __LINE__);
+
 	BCM_REFERENCE(i);
 	BCM_REFERENCE(adapter);
 	/* Sanity check on the module parameters
@@ -755,7 +757,6 @@ static int dhd_wifi_platform_load_sdio(void)
 		return -EINVAL;
 
 #if defined(BCMLXSDMMC) && !defined(DHD_PRELOAD)
-	printk(KERN_DEBUG"%s:%d", __func__, __LINE__);
 	if (dhd_wifi_platdata == NULL) {
 		DHD_ERROR(("DHD wifi platform data is required for Android build\n"));
 		return -EINVAL;
@@ -812,7 +813,6 @@ static int dhd_wifi_platform_load_sdio(void)
 		}
 
 	}
-	printk(KERN_DEBUG"%s:%d", __func__, __LINE__);
 
 	err = dhd_bus_register();
 
@@ -834,7 +834,6 @@ static int dhd_wifi_platform_load_sdio(void)
 		goto fail;
 	}
 
-	printk(KERN_DEBUG"%s:%d\n", __func__, __LINE__);
 	return err;
 
 fail:
@@ -845,7 +844,7 @@ fail:
 		wifi_platform_bus_enumerate(adapter, FALSE);
 	}
 #else
-	printk(KERN_DEBUG"%s:%d", __func__, __LINE__);
+
 	/* x86 bring-up PC needs no power-up operations */
 	err = dhd_bus_register();
 
